import Cache from './Cache';
import Hash from './Hash';
import { AxiosInstance } from 'axios';
/**
 * An Api instance defines the properties to consume a restful api
 */
export default class Api {
    /**
     * Default base path
     */
    static base: string;
    /**
     * The base path to your Api
     * Eg.: http://my-api.com
     */
    base: string;
    /**
     * Default headers
     */
    static headers: Hash<string>;
    /**
     * Api headers. You can use it for Authentication for example.
     */
    headers: Hash<string>;
    /**
     * Default use cache value
     */
    static useCache: boolean;
    /**
     * Sets whether or not caching is enabled
     */
    useCache: boolean;
    /**
     * Default use promise cache value
     */
    static usePromiseCache: boolean;
    /**
     * If set to true, the promises to fetch data will be cached,
     * so if you do the same request twice, before the results come back, the response will be the same Promise.
     */
    usePromiseCache: boolean;
    /**
     * Cache instance attached to the Api
     */
    cache: Cache;
    constructor(properties?: Hash<any>);
    /**
     * Get path.
     * This is very useful if you have one Api for each model for example
     * Eg. books (base + path would look like http://my-api.com/books)
     * @type {string}
     */
    path(): string;
    /**
     * Get api base
     * @return {string}
     */
    resolveBase(): string;
    /**
     * Get api headers
     * @return {Hash<string>}
     */
    resolveHeaders(): Hash<string>;
    /**
     * Get base url (base + path)
     * @returns {string}
     */
    baseUrl(): string;
    /**
     * Check if instance is using cache
     */
    isUsingCache(): boolean;
    /**
     * Check if instance is using promise cache
     */
    isUsingPromiseCache(): boolean;
    /**
     * Get http requester
     * @return {AxiosInstance}
     */
    http(): AxiosInstance;
    /**
     * Cache promise by name, so it doesn't get executed again before the result comes back
     */
    cachePromise(name: string, fn: (resolve: (value?: any) => void, reject: (reason?: any) => void) => void): Promise<any>;
    /**
     * Add JWT authorization header for a given token
     */
    static setJwtToken(token: string, target?: Hash<any>): void;
    /**
     * Add JWT authorization header for a given token
     */
    setJwtToken(token: string): void;
    /**
     * Remove JWT authorization header
     */
    static removeJwtToken(target?: Hash<any>): void;
    /**
     * Remove JWT authorization header
     */
    removeJwtToken(): void;
    /**
     * Add CSRF token to static header
     */
    static setCsrfToken(token: string, target?: Hash<any>): void;
    setCsrfToken(token: string): void;
    /**
     * Remove CSRF token from static header
     */
    static removeCsrfToken(target?: Hash<any>): void;
    /**
     * Remove CSRF token from header
     */
    removeCsrfToken(): void;
}
