"use strict";
var Cache_1 = require("./Cache");
var axios_1 = require("axios");
/**
 * An Api instance defines the properties to consume a restful api
 */
var Api = (function () {
    function Api(properties) {
        if (properties === void 0) { properties = {}; }
        /**
         * Cache instance attached to the Api
         */
        this.cache = new Cache_1.default(this);
        Object.assign(this, properties);
    }
    /**
     * Get path.
     * This is very useful if you have one Api for each model for example
     * Eg. books (base + path would look like http://my-api.com/books)
     * @type {string}
     */
    Api.prototype.path = function () {
        return '';
    };
    /**
     * Get api base
     * @return {string}
     */
    Api.prototype.resolveBase = function () {
        var self = this.constructor;
        return this.base || self.base || Api.base;
    };
    /**
     * Get api headers
     * @return {Hash<string>}
     */
    Api.prototype.resolveHeaders = function () {
        var self = this.constructor;
        return this.headers || self.headers || Api.headers;
    };
    /**
     * Get base url (base + path)
     * @returns {string}
     */
    Api.prototype.baseUrl = function () {
        var base = this.resolveBase();
        if (!base.endsWith('/')) {
            base += '/';
        }
        var path = this.path();
        if (!path.endsWith('/')) {
            path += '/';
        }
        return base + path;
    };
    /**
     * Check if instance is using cache
     */
    Api.prototype.isUsingCache = function () {
        return this.useCache || this.constructor.useCache || Api.useCache;
    };
    /**
     * Check if instance is using promise cache
     */
    Api.prototype.isUsingPromiseCache = function () {
        return this.usePromiseCache || this.constructor.usePromiseCache || Api.usePromiseCache;
    };
    /**
     * Get http requester
     * @return {AxiosInstance}
     */
    Api.prototype.http = function () {
        return axios_1.default.create({
            baseURL: this.baseUrl(),
            headers: this.resolveHeaders()
        });
    };
    /**
     * Cache promise by name, so it doesn't get executed again before the result comes back
     */
    Api.prototype.cachePromise = function (name, fn) {
        var _this = this;
        if (!this.isUsingPromiseCache()) {
            return new Promise(fn);
        }
        var cached = this.cache.getPromise(name);
        if (cached) {
            return cached;
        }
        var promise = new Promise(fn);
        this.cache.setPromise(name, promise);
        var destroyCallback = function () {
            _this.cache.destroyPromise(name);
        };
        promise.then(destroyCallback, destroyCallback);
        return promise;
    };
    /**
     * Add JWT authorization header for a given token
     */
    Api.setJwtToken = function (token, target) {
        if (target === void 0) { target = this.headers; }
        target['Authorization'] = "Bearer " + token;
    };
    /**
     * Add JWT authorization header for a given token
     */
    Api.prototype.setJwtToken = function (token) {
        var self = this.constructor;
        self.setJwtToken(token, this.headers);
    };
    /**
     * Remove JWT authorization header
     */
    Api.removeJwtToken = function (target) {
        if (target === void 0) { target = this.headers; }
        delete target['Authorization'];
    };
    /**
     * Remove JWT authorization header
     */
    Api.prototype.removeJwtToken = function () {
        var self = this.constructor;
        self.removeJwtToken(this.headers);
    };
    /**
     * Add CSRF token to static header
     */
    Api.setCsrfToken = function (token, target) {
        if (target === void 0) { target = this.headers; }
        target['X-CSRF-Token'] = token;
    };
    Api.prototype.setCsrfToken = function (token) {
        var self = this.constructor;
        self.setCsrfToken(token, this.headers);
    };
    /**
     * Remove CSRF token from static header
     */
    Api.removeCsrfToken = function (target) {
        if (target === void 0) { target = this.headers; }
        delete target['X-CSRF-Token'];
    };
    /**
     * Remove CSRF token from header
     */
    Api.prototype.removeCsrfToken = function () {
        var self = this.constructor;
        self.removeCsrfToken(this.headers);
    };
    return Api;
}());
/**
 * Default headers
 */
Api.headers = {};
/**
 * Default use cache value
 */
Api.useCache = true;
/**
 * Default use promise cache value
 */
Api.usePromiseCache = true;
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Api;
//# sourceMappingURL=Api.js.map