"use strict";
/**
 * The cache is responsible for storing objects, lists and promises
 * The goal is to optimize the application speed and network usage
 */
var Cache = (function () {
    function Cache(api) {
        /**
         * The list of objects cached.
         */
        this.objects = {};
        /**
         * The list of lists of object cached.
         */
        this.lists = {};
        /**
         * The list of promises cached.
         */
        this.promises = {};
        this.api = api;
    }
    /**
     * Get key from value
     */
    Cache.resolveKey = function (value) {
        if (typeof value === 'object') {
            return value.key().toString();
        }
        return value.toString();
    };
    /**
     * Check if caching is enabled
     */
    Cache.prototype.enabled = function () {
        return !this.api || this.api.isUsingCache();
    };
    /**
     * Check if promise caching is enabled
     */
    Cache.prototype.promiseEnabled = function () {
        return !this.api || this.api.isUsingPromiseCache();
    };
    /**
     * Get object
     */
    Cache.prototype.get = function (value) {
        if (this.enabled()) {
            return this.objects[value.toString()];
        }
    };
    /**
     * Get object by property value
     */
    Cache.prototype.where = function (property, value) {
        var _this = this;
        if (this.enabled()) {
            var values = Object.keys(this.objects).map(function (key) { return _this.objects[key]; });
            return values.find(function (object) {
                return object[property] === value;
            });
        }
    };
    /**
     * Set object
     */
    Cache.prototype.set = function (value, clearLists) {
        var _this = this;
        if (clearLists === void 0) { clearLists = true; }
        if (Array.isArray(value)) {
            var list = value;
            return list.map(function (it) { return _this.set(it, clearLists); });
        }
        var object = value;
        if (this.enabled()) {
            this.objects[object.key()] = object;
            if (clearLists) {
                this.lists = {};
            }
        }
        return object;
    };
    /**
     * Remove object
     */
    Cache.prototype.destroy = function (value) {
        var self = this.constructor;
        delete this.objects[self.resolveKey(value)];
        this.lists = {};
    };
    /**
     * Get list by name
     */
    Cache.prototype.getList = function (name) {
        if (this.enabled()) {
            return this.lists[name];
        }
    };
    /**
     * Set list by name
     */
    Cache.prototype.setList = function (name, list) {
        if (this.enabled()) {
            this.lists[name] = list;
        }
        return list;
    };
    Cache.prototype.destroyList = function (name) {
        delete this.lists[name];
    };
    /**
     * Get promise by name
     */
    Cache.prototype.getPromise = function (name) {
        if (this.promiseEnabled()) {
            return this.promises[name];
        }
    };
    /**
     * Set promise by name
     */
    Cache.prototype.setPromise = function (name, promise) {
        if (this.promiseEnabled()) {
            return this.promises[name] = promise;
        }
        return promise;
    };
    /**
     * Destroy promise by name
     */
    Cache.prototype.destroyPromise = function (name) {
        delete this.promises[name];
    };
    return Cache;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Cache;
//# sourceMappingURL=Cache.js.map