import Store, { IStoreForeign, IStoreFetch, IStoreSet, IStoreAct } from './Store';
import Hash from './Hash';
export interface IModel {
    key(): string;
}
/**
 * A Model is usually relative to a table in the database.
 */
export default class Model implements IModel {
    /**
     * The path associated with the model
     */
    static path: string;
    /**
     * The singular name associated with the model
     * This is used for finding variable names when working with FKs
     */
    static singular: string;
    /**
     * The Store associated with the model
     * If none is given and a request is made, a new store is auto generated with the given `storeProperties`
     */
    static store: Store<Model>;
    /**
     * The name of the primary key column
     */
    static keyColumn: string;
    /**
     * This variable is used to set the default store properties
     * If you're using a custom store, don't worry about this
     */
    static storeProperties: Hash<any>;
    /**
     * Get model store
     */
    store(): Store<this>;
    /**
     * Get an instance of store
     */
    static resolveStore(properties?: Hash<any>): Store<Model>;
    /**
     * Create new instance of store
     */
    static createStore(properties?: Hash<any>): Store<Model>;
    /**
     * Get value of primary key
     */
    key(): any;
    /**
     * Get mapped properties with values of model
     * The shallow map does not contain objects (consequently, does not contain FKs)
     */
    map(denormalize?: boolean): Hash<any>;
    /**
     * Get foreign key value for a given model
     */
    foreignKey(model: typeof Model): any;
    /**
     * Get foreign key name for a given model
     */
    static foreignKey(): string;
    /**
     * Camelize transformer name
     */
    private static camelizeTransformer(value);
    /**
     * Resolve normalizing function
     */
    static resolveTransformer(key: string, type?: string): string;
    /**
     * Normalize property
     */
    static normalize(key: string, value: any): any;
    /**
     * Denormalize property
     */
    static denormalize(key: string, value: any): any;
    /**
     * Set values to instance
     */
    set(values: Hash<any>, normalize?: boolean): this;
    /**
     * Duplicate model
     */
    duplicate(): this;
    /**
     * Create one or an array of model instances
     */
    static make(values: Hash<any> | Hash<any>[], normalize?: boolean): Model | Model[];
    /**
     * Reload model instance
     */
    reload(): Promise<this>;
    /**
     * Find model object by id
     */
    static find(key: any, options?: IStoreFetch): Promise<Model>;
    /**
     * Persist model
     */
    save(options?: IStoreSet): Promise<this>;
    /**
     * Create and persist one or more instances
     */
    static save(values: Hash<any>, options?: IStoreSet): Promise<Model> | Promise<Model>[];
    /**
     * Destroy model
     */
    destroy(options?: IStoreSet): Promise<any>;
    /**
     * Destroy
     */
    static destroy(key: any, options?: IStoreSet): Promise<any> | Promise<any>[];
    /**
     * Get promise to belongsTo FK relation
     */
    belongsTo(model: typeof Model, options?: IStoreFetch): Promise<Model>;
    /**
     * Get promise to belongsToMany FK relation
     */
    belongsToMany(model: typeof Model, options?: IStoreForeign): Promise<this[]>;
    /**
     * Get promise to belongsToMany FK relation
     */
    static belongsToMany(model: typeof Model, value: any, options?: IStoreForeign): Promise<Model[]>;
    /**
     * Get promise to hasOne FK relation
     */
    hasOne(model: typeof Model, options?: IStoreForeign): Promise<Model>;
    /**
     * Resolve first promise result or null
     */
    protected static resolveFirst(promise: Promise<Model[]>): Promise<Model>;
    /**
     * Get promise to hasOne FK relation statically
     */
    static hasOne(model: typeof Model, key: any, options?: IStoreForeign): Promise<Model>;
    /**
     * Get promise to hasMany FK relation
     */
    hasMany(model: typeof Model, options?: IStoreForeign): Promise<Model[]>;
    /**
     * Get promise to hasMany FK relation statically
     */
    static hasMany(model: typeof Model, key: any, options?: IStoreForeign): Promise<Model[]>;
    /**
     * Call action for model
     */
    act(action: string, options?: IStoreAct): Promise<any>;
    /**
     * Call action
     */
    static act(key: any, action: string, options?: IStoreAct): Promise<any>;
    /**
     * Find model object by property value
     */
    static where(property: string, key: any, options?: IStoreAct): Promise<Model>;
    /**
     * Get list of stores
     */
    static all(options?: IStoreFetch): Promise<Model[]>;
    /**
     * Get singularized path
     * If the `singular` variable is not defined, guess it from the `path`
     */
    static singularized(): string;
}
