"use strict";
var Store_1 = require("./Store");
/**
 * A Model is usually relative to a table in the database.
 */
var Model = (function () {
    function Model() {
    }
    /**
     * Get model store
     */
    Model.prototype.store = function () {
        var self = this.constructor;
        return self.resolveStore();
    };
    /**
     * Get an instance of store
     */
    Model.resolveStore = function (properties) {
        if (properties === void 0) { properties = undefined; }
        return this.store = this.store || this.createStore(properties);
    };
    /**
     * Create new instance of store
     */
    Model.createStore = function (properties) {
        if (properties === void 0) { properties = this.storeProperties; }
        return new Store_1.default(this, properties);
    };
    /**
     * Get value of primary key
     */
    Model.prototype.key = function () {
        var self = this.constructor;
        return this[self.keyColumn];
    };
    /**
     * Get mapped properties with values of model
     * The shallow map does not contain objects (consequently, does not contain FKs)
     */
    Model.prototype.map = function (denormalize) {
        var _this = this;
        if (denormalize === void 0) { denormalize = true; }
        var self = this.constructor;
        var map = {};
        Object.keys(this).forEach(function (key) {
            var value = _this[key];
            if (denormalize) {
                value = self.denormalize(key, value);
            }
            if (typeof value !== 'object') {
                map[key] = value;
            }
        });
        return map;
    };
    /**
     * Get foreign key value for a given model
     */
    Model.prototype.foreignKey = function (model) {
        return this[model.foreignKey()];
    };
    /**
     * Get foreign key name for a given model
     */
    Model.foreignKey = function () {
        return this.singularized() + "_" + this.keyColumn;
    };
    /**
     * Camelize transformer name
     */
    Model.camelizeTransformer = function (value) {
        return value.split(new RegExp('[_|-]')).map(function (part) {
            return part.substring(0, 1).toUpperCase() + part.substring(1);
        }).join('');
    };
    /**
     * Resolve normalizing function
     */
    Model.resolveTransformer = function (key, type) {
        if (type === void 0) { type = 'normalize'; }
        var underscored = type + "_" + key;
        if (this[underscored]) {
            return underscored;
        }
        var camel = type + this.camelizeTransformer(key);
        if (this[camel]) {
            return camel;
        }
    };
    /**
     * Normalize property
     */
    Model.normalize = function (key, value) {
        var method = this.resolveTransformer(key);
        return method ? this[method](value) : value;
    };
    /**
     * Denormalize property
     */
    Model.denormalize = function (key, value) {
        var method = this.resolveTransformer(key, 'denormalize');
        return method ? this[method](value) : value;
    };
    /**
     * Set values to instance
     */
    Model.prototype.set = function (values, normalize) {
        var _this = this;
        if (normalize === void 0) { normalize = false; }
        var self = this.constructor;
        Object.keys(values).forEach(function (key) {
            var value = values[key];
            _this[key] = normalize ? self.normalize(key, value) : value;
        });
        return this;
    };
    /**
     * Duplicate model
     */
    Model.prototype.duplicate = function () {
        var self = this.constructor;
        var map = this.map(false);
        delete map[self.keyColumn];
        return self.make(map);
    };
    /**
     * Create one or an array of model instances
     */
    Model.make = function (values, normalize) {
        var _this = this;
        if (normalize === void 0) { normalize = false; }
        if (Array.isArray(values)) {
            return values.map(function (it) { return _this.make(it); });
        }
        return new this().set(values, normalize);
    };
    /**
     * Reload model instance
     */
    Model.prototype.reload = function () {
        var self = this.constructor;
        return self.find(this.key());
    };
    /**
     * Find model object by id
     */
    Model.find = function (key, options) {
        if (options === void 0) { options = undefined; }
        return this.resolveStore().get(key, options);
    };
    /**
     * Persist model
     */
    Model.prototype.save = function (options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.save(this.map(), options);
    };
    /**
     * Create and persist one or more instances
     */
    Model.save = function (values, options) {
        var _this = this;
        if (options === void 0) { options = undefined; }
        if (Array.isArray(values)) {
            return values.map(function (it) { return _this.save(it); });
        }
        if (values[this.keyColumn]) {
            return this.resolveStore().update(values, options);
        }
        return this.resolveStore().create(values, options);
    };
    /**
     * Destroy model
     */
    Model.prototype.destroy = function (options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.destroy(this.key(), options);
    };
    /**
     * Destroy
     */
    Model.destroy = function (key, options) {
        var _this = this;
        if (options === void 0) { options = undefined; }
        if (Array.isArray(key)) {
            return key.map(function (it) { return _this.destroy(it); });
        }
        return this.resolveStore().destroy(key, options);
    };
    /**
     * Get promise to belongsTo FK relation
     */
    Model.prototype.belongsTo = function (model, options) {
        if (options === void 0) { options = undefined; }
        return model.find(this.foreignKey(model), options);
    };
    /**
     * Get promise to belongsToMany FK relation
     */
    Model.prototype.belongsToMany = function (model, options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.belongsToMany(model, this.key(), options);
    };
    /**
     * Get promise to belongsToMany FK relation
     */
    Model.belongsToMany = function (model, value, options) {
        if (options === void 0) { options = undefined; }
        return this.hasMany(model, value, options);
    };
    /**
     * Get promise to hasOne FK relation
     */
    Model.prototype.hasOne = function (model, options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.hasOne(model, this.key(), options);
    };
    /**
     * Resolve first promise result or null
     */
    Model.resolveFirst = function (promise) {
        return new Promise(function (resolve, reject) {
            promise.then(function (objects) {
                resolve(objects.length ? objects[0] : null);
            }, reject);
        });
    };
    /**
     * Get promise to hasOne FK relation statically
     */
    Model.hasOne = function (model, key, options) {
        if (options === void 0) { options = undefined; }
        var promise = this.hasMany(model, key, options);
        return this.resolveFirst(promise);
    };
    /**
     * Get promise to hasMany FK relation
     */
    Model.prototype.hasMany = function (model, options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.hasMany(model, this.key(), options);
    };
    /**
     * Get promise to hasMany FK relation statically
     */
    Model.hasMany = function (model, key, options) {
        if (options === void 0) { options = undefined; }
        return this.resolveStore().foreign(model, key, options);
    };
    /**
     * Call action for model
     */
    Model.prototype.act = function (action, options) {
        if (options === void 0) { options = undefined; }
        var self = this.constructor;
        return self.act(this.key(), action, options);
    };
    /**
     * Call action
     */
    Model.act = function (key, action, options) {
        if (options === void 0) { options = undefined; }
        return this.resolveStore().act(key, action, options);
    };
    /**
     * Find model object by property value
     */
    Model.where = function (property, key, options) {
        if (options === void 0) { options = undefined; }
        return this.resolveStore().where(property, key, options);
    };
    /**
     * Get list of stores
     */
    Model.all = function (options) {
        if (options === void 0) { options = undefined; }
        return this.resolveStore().index(options);
    };
    /**
     * Get singularized path
     * If the `singular` variable is not defined, guess it from the `path`
     */
    Model.singularized = function () {
        return this.singular || this.path.substring(0, this.path.length - 1);
    };
    return Model;
}());
/**
 * The name of the primary key column
 */
Model.keyColumn = 'id';
/**
 * This variable is used to set the default store properties
 * If you're using a custom store, don't worry about this
 */
Model.storeProperties = {};
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Model;
//# sourceMappingURL=Model.js.map