"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Api_1 = require("./Api");
var Cache_1 = require("./Cache");
/**
 * A Store is an Api that handles the requests for a given Model
 */
var Store = (function (_super) {
    __extends(Store, _super);
    function Store(model, properties) {
        if (properties === void 0) { properties = {}; }
        var _this = _super.call(this, properties) || this;
        _this.model = model;
        if (model && !model.store) {
            model.store = _this;
        }
        return _this;
    }
    /**
     * @inheritDoc
     */
    Store.prototype.path = function () {
        return this.model.path;
    };
    /**
     * Check if method is allowed
     */
    Store.prototype.isMethodAllowed = function (method) {
        if (this.denies && this.denies.indexOf(method) !== -1) {
            return false;
        }
        return !this.allows || this.allows.indexOf(method) !== -1;
    };
    /**
     * Verify if a method can be executed. If it can't, throw an error
     * @throws Error
     */
    Store.prototype.verifyPermission = function (method) {
        if (!this.isMethodAllowed(method)) {
            var message = "Method '" + method + "' is not allowed in '" + this.path() + "' store";
            throw new Error(message);
        }
    };
    /**
     * Get new model instance
     */
    Store.prototype.make = function (values, normalize) {
        if (normalize === void 0) { normalize = true; }
        return this.model.make(values, normalize);
    };
    /**
     * Get index
     */
    Store.prototype.index = function (options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        if (!options.permit) {
            this.verifyPermission('index');
        }
        var url = options.url || '';
        return this.cachePromise("index/" + url, function (resolve, reject) {
            var list = _this.cache.getList("index/" + url);
            if (!options.ignoreCache && list) {
                if (list) {
                    return resolve(list);
                }
            }
            var promise = _this.http().get(url);
            promise.then(function (response) {
                var list = _this.make(response.data);
                _this.cache.setList("index/" + url, list);
                if (_this.setIndex) {
                    _this.cache.set(list, false);
                }
                resolve(list);
            }, reject);
        });
    };
    /**
     * Make http get request
     */
    Store.prototype.get = function (key, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        if (!options.permit) {
            this.verifyPermission('get');
        }
        key = Cache_1.default.resolveKey(key);
        var url = options.url || key.toString();
        return this.cachePromise("get/" + url, function (resolve, reject) {
            if (!options.ignoreCache) {
                var object = _this.cache.get(key);
                if (object) {
                    return resolve(object);
                }
            }
            var promise = _this.http().get(url);
            promise.then(function (response) {
                var instance = _this.make(response.data);
                resolve(_this.cache.set(instance, false));
            }, reject);
        });
    };
    /**
     * Make http get request to fetch by property
     */
    Store.prototype.where = function (property, value, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        if (!options.permit) {
            this.verifyPermission('where');
        }
        var url = options.url;
        if (!options.url) {
            url = property + "/" + value;
        }
        return this.cachePromise("where/" + url, function (resolve, reject) {
            if (!options.ignoreCache) {
                var object = _this.cache.where(property, value);
                if (object) {
                    return resolve(object);
                }
            }
            var promise = _this.http().get(url);
            promise.then(function (response) {
                var instance = _this.make(response.data);
                resolve(_this.cache.set(instance, false));
            }, reject);
        });
    };
    /**
     * Make http post request
     */
    Store.prototype.create = function (data, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.verifyPermission('create');
        var promise = this.http().post(options.url || '', data);
        return new Promise(function (resolve, reject) {
            promise.then(function (response) {
                var instance = _this.make(response.data);
                resolve(_this.cache.set(instance));
            }, reject);
        });
    };
    /**
     * Make http put request
     */
    Store.prototype.update = function (data, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.verifyPermission('update');
        return new Promise(function (resolve, reject) {
            var promise = _this.http().put(options.url || data[_this.model.keyColumn].toString(), data);
            promise.then(function (response) {
                var instance = _this.make(response.data);
                resolve(_this.cache.set(instance));
            }, reject);
        });
    };
    /**
     * Make http request to destroy model
     */
    Store.prototype.destroy = function (value, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        this.verifyPermission('destroy');
        var id = Cache_1.default.resolveKey(value);
        var promise = this.http().delete(options.url || id);
        promise.then(function () { return _this.cache.destroy(id); });
        return promise;
    };
    /**
     * Call foreign method of store associated with given model
     */
    Store.prototype.by = function (model, key, options) {
        if (options === void 0) { options = {}; }
        return model.resolveStore().foreign(this.model, key, options);
    };
    /**
     * Make http request to fetch instances by foreign key
     */
    Store.prototype.foreign = function (model, key, options) {
        var _this = this;
        if (options === void 0) { options = {}; }
        if (!options.permit) {
            this.verifyPermission('foreign');
        }
        key = Cache_1.default.resolveKey(key);
        var url = options.url || key + "/" + model.path;
        return this.cachePromise("foreign/" + url, (function (resolve, reject) {
            var list = _this.cache.getList(url);
            if (!options.ignoreCache && list) {
                return resolve(list);
            }
            _this.http().get(url).then(function (response) {
                var list = _this.make(response.data);
                _this.cache.setList(url, list);
                if (options.set || (!Array.isArray(list) && options.set !== false)) {
                    _this.cache.set(list, false);
                }
                resolve(list);
            }, reject);
        }));
    };
    /**
     * Make http to act
     */
    Store.prototype.act = function (key, action, options) {
        if (options === void 0) { options = {}; }
        if (!options.permit) {
            this.verifyPermission('act');
        }
        key = Cache_1.default.resolveKey(key);
        var url = options.url || key + "/" + action;
        var method = this.http()[options.method || 'put'];
        return method(url, options.data);
    };
    return Store;
}(Api_1.default));
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Store;
//# sourceMappingURL=Store.js.map